import sys
import argparse
import karnak.util.athena as ka
import karnak.util.log as klog

def run(args):
    parser = argparse.ArgumentParser(description='karnak - aws athena query test')
    parser.add_argument('sql', type=str)
    parser.add_argument('--database', type=str)
    parser.add_argument('--region', type=str, default='us-east-1')
    parser.add_argument('--workgroup', type=str, default='primary')
    parser.add_argument('--method', type=str, choices=['rest', 'jdbc', 'csv'], default='rest')
    parser.add_argument('--verbosity', '-v', type=int, default=1,
                        help='verbosity level 0-5, default 1, larger means more verbose')
    args = parser.parse_args(args)
    klog.set_log_level(args.verbosity)

    df = ka.select_pd(sql=args.sql, aws_region=args.region, database=args.database,
                      workgroup=args.workgroup, method=args.method)
    print(df)

if __name__ == "__main__":
    run(sys.argv[1:])



import os
import errno
from typing import List


def path_concat(path: str, *elements: str) -> str:
    base_path = path.rstrip('/')
    safe_elements = [base_path] + [e.rstrip('/').lstrip('/') for e in elements if
                                   e is not None and e.rstrip('/').lstrip('/') != '']
    return '/'.join(safe_elements)


def create_folder(filepath: str) -> None:
    if not os.path.exists(os.path.dirname(filepath)):
        try:
            os.makedirs(os.path.dirname(filepath))
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise



import argparse
import datetime
import dateutil.parser
import pytz
import os


def argparse_date(date_str):
    try:
        return datetime.datetime.strptime(date_str, '%Y-%m-%d')
    except ValueError:
        raise argparse.ArgumentTypeError(f'date {date_str} not in format YYYY-MM-DD')


def argparse_timestamp(s):
    try:
        return dateutil.parser.isoparse(s)
    except Exception:
        raise argparse.ArgumentTypeError(f'not a valid timestamp: {s}')


def argparse_timezone(s):
    try:
        return pytz.timezone(s)
    except Exception:
        raise argparse.ArgumentTypeError(f'not a valid timezone: {s}')


def argparse_dir_path(path):
    if os.path.isdir(path):
        return path
    else:
        raise argparse.ArgumentTypeError(f"{path} is not a valid path")


def argparse_file_path(path):
    if os.path.isfile(path):
        return path
    else:
        raise argparse.ArgumentTypeError(f"{path} is not a valid file")



import karnak.util.log as klog

import pandas as pd
import contextlib
import pyathena
import pyathena.pandas.util
from pyathena.pandas.cursor import PandasCursor
import pyathenajdbc
import pyathenajdbc.util
from typing import Optional


def _select_pd_jdbc(sql: str, aws_region: str, database: Optional[str] = None,
                    workgroup: Optional[str] = None, s3_output_location: Optional[str] = None) -> pd.DataFrame:
    klog.trace('running query on athena, method jdbc: {}', ' '.join(sql.split()))
    if klog.log_level > 0:
        klog.debug('running query on athena, method rest')

    params = {'Workgroup': workgroup,
              'AwsRegion': aws_region,
              'S3OutputLocation': s3_output_location}
    if database is not None:
        params['Schema'] = database

    with contextlib.closing(pyathenajdbc.connect(**params)) as conn:
        with contextlib.closing(conn.cursor()) as cursor:
            results = cursor.execute(sql)
            klog.trace('query executed')

            if klog.log_level > 0:
                klog.debug('query execution completed.')
            df = pyathenajdbc.util.as_pandas(results)
            klog.trace('query results read into dataframe with {} rows.', len(df))
            return df


def _select_pd_rest(sql: str, aws_region: str, database=None, workgroup=None, s3_output_location=None,
                    method='rest') -> pd.DataFrame:
    assert method in ['rest', 'csv']
    klog.trace('running query on athena, method {}: {}', method, ' '.join(sql.split()))
    if klog.log_level > 0:
        klog.debug('running query on athena, method {}', method)

    params = {'work_group': workgroup,
              'region_name': aws_region,
              'output_location': s3_output_location}
    if database is not None:
        params['schema_name'] = database
    if method == 'csv':
        params['cursor_class'] = PandasCursor

    with contextlib.closing(pyathena.connect(**params)) as conn:
        with contextlib.closing(conn.cursor()) as cursor:
            results = cursor.execute(sql)
            klog.trace('query stats: data scanned: {:.2f} MB, total query time {:.3f}s'.format(
                results.data_scanned_in_bytes / (1024 * 1024.0),
                results.total_execution_time_in_millis / 1000.0))

            if klog.log_level > 0:
                klog.debug('query execution completed.')
            if method == 'csv':
                df = results.as_pandas()
            else:
                df = pyathena.pandas.util.as_pandas(results)
            klog.trace('query results converted to dataframe with {} rows.', len(df))
            return df


def select_pd(sql: str, aws_region: str, database: Optional[str] = None,
              workgroup: Optional[str] = None, s3_output_location: Optional[str] = None,
              method: str = 'rest') -> pd.DataFrame:
    """Runs an sql query in AWS Athena and returns results as pandas dataframe.

        Params:
            sql: sql query which will be executed by Athena.
            database: athena database used as default (optional if database is explicit in the sql query)
            workgroup: athena database. Either workgroup ou output_path (or both) must be provided.
            s3_output_path: athena s3 output path.
            aws_region: aws s3 region name, e.g.: 'us_east-1'
            method: results download method. 'rest' or 'jdbc' are best suited for small results data;
                'csv' is suitable for larger results data.
    """

    assert method in ['jdbc', 'rest', 'csv']
    if method in ['rest', 'csv']:
        return _select_pd_rest(sql=sql, aws_region=aws_region, database=database,
                               workgroup=workgroup, s3_output_location=s3_output_location, method=method)
    else:  # 'jdbc'
        return _select_pd_jdbc(sql=sql, aws_region=aws_region, database=database,
                               workgroup=workgroup, s3_output_location=s3_output_location)


def test_fixture_pd(aws_region: str, workgroup: Optional[str] = None, s3_output_location: Optional[str] = None,
                    method: str = 'rest'):
    sql = """
        SELECT * FROM (VALUES
            (CAST ('str1' AS VARCHAR), 1, CAST(1 AS BIGINT), 1.0, TRUE, '["elem1", "elem2"]'),
            ('str2', 2, 2,  2.0, FALSE, '["elem3"]'),
            ('', 0, 0, 0.0, FALSE, '[]'),
            (NULL, NULL, NULL, NULL, NULL, NULL)
        )
            x(c_str, c_int, c_bigint, c_double, c_boolean, c_json_list);
"""
    return select_pd(sql, aws_region=aws_region, workgroup=workgroup, s3_output_location=s3_output_location,
                     method=method)



import datetime
import time
import karnak.util.log as klog
import psutil


class KTimer:
    def __init__(self):
        self.start_time = time.time()
        self.last_time = self.start_time

    def log_timer(self, message=None, end_time=None):
        if message is None:
            message = 'elapsed'
        klog.debug('{} {}', message, self.get_elapsed_str(end_time))

    def get_elapsed(self, end_time=None):
        if end_time is None:
            end_time = time.time()
        return datetime.timedelta(seconds=end_time - self.start_time)

    def get_elapsed_str(self, end_time=None):
        elapsed = self.get_elapsed(end_time)
        return str(elapsed)

    def get_delta_elapsed(self, end_time=None):
        if end_time is None:
            end_time = time.time()
        ret = datetime.timedelta(seconds=end_time - self.last_time)
        self.last_time = end_time
        return ret

    def get_delta_elapsed_str(self, end_time=None):
        elapsed = self.get_delta_elapsed(end_time)
        return str(elapsed)


class KProfiler(KTimer):
    def __init__(self):
        mem_usage = psutil.virtual_memory()
        self.start_mem_available = mem_usage.available
        self.start_mem_used = mem_usage.used
        self.start_mem_free = mem_usage.free
        self.start_mem_total = mem_usage.total

        self.last_mem_available = mem_usage.available
        self.last_mem_used = mem_usage.used
        self.last_mem_free = mem_usage.free
        self.last_mem_total = mem_usage.total
        super().__init__()

    @staticmethod
    def pretty_mem(size_bytes, round_decimals=3) -> str:
        size = size_bytes
        for unit in ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB']:
            if abs(size) < 1024.0 or unit == 'PiB':
                break
            size = size / 1024.0
        return f"{size:.{round_decimals}f} {unit}"

    @staticmethod
    def log_mem(message=None):
        pretty_mem = KProfiler.pretty_mem
        mem_usage = psutil.virtual_memory()
        msg = ''
        if message is not None:
            msg = message + ': '
        klog.debug(f'{msg}mem avail {pretty_mem(mem_usage.available)}, '
                   f'used {pretty_mem(mem_usage.available)}, '
                   f'free {pretty_mem(mem_usage.free)}, '
                   f'total {pretty_mem( mem_usage.total)}')

    def log_delta(self, message=None, end_time=None):
        msg = ''
        if message is not None:
            msg = message + ': '

        delta_elapsed_str = self.get_delta_elapsed_str(end_time)
        total_elapsed_str = self.get_elapsed_str(end_time)
        mem_usage = psutil.virtual_memory()

        delta_avail_str = self.pretty_mem(mem_usage.available - self.last_mem_available)
        delta_used_str = self.pretty_mem(mem_usage.used - self.last_mem_used)

        klog.debug(f'{msg}elapsed {delta_elapsed_str}s, '
                   f'total elapsed {total_elapsed_str}, '
                   f'mem: delta avail {delta_avail_str}, '
                   f'delta used {delta_used_str}, '
                   f'avail {self.pretty_mem(mem_usage.available)}, '
                   f'used {self.pretty_mem(mem_usage.used)}, '
                   f'free {self.pretty_mem(mem_usage.free)}, '
                   f'total {self.pretty_mem(mem_usage.total)}')

        self.last_mem_available = mem_usage.available
        self.last_mem_used = mem_usage.used
        self.last_mem_free = mem_usage.free
        self.last_mem_total = mem_usage.total

    def log_cumulative(self, message=None, end_time=None):
        msg = ''
        if message is not None:
            msg = message + ': '

        elapsed_str = self.get_elapsed_str(end_time)
        mem_usage = psutil.virtual_memory()
        delta_avail_str = self.pretty_mem(mem_usage.available - self.start_mem_available)
        delta_used_str = self.pretty_mem(mem_usage.used - self.start_mem_used)

        klog.debug(f'{msg}elapsed {elapsed_str}s, '
                   f'mem: delta avail {delta_avail_str}, '
                   f'delta used {delta_used_str}, '
                   f'avail {self.pretty_mem(mem_usage.available)}, '
                   f'used {self.pretty_mem(mem_usage.used)}, '
                   f'free {self.pretty_mem(mem_usage.free)}, '
                   f'total {self.pretty_mem(mem_usage.total)}')

    def log_profile(self, message=None, end_time=None):
        self.log_cumulative(message=message, end_time=end_time)



import setuptools
import pathlib
import pkg_resources

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def requirements():
    with pathlib.Path('requirements.txt').open() as requirements_txt:
        install_requires = [
            str(requirement)
            for requirement
            in pkg_resources.parse_requirements(requirements_txt)
        ]
    return install_requires


setuptools.setup(
    name="karnak",
    version="0.1.6",
    author="Leonardo Rossi",
    author_email="leorossi@cogitaslabs.com",
    description="Karnak Data Platform Community Libraries",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cogitaslabs/karnak-community",
    packages=setuptools.find_namespace_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=requirements()
)


import sys
import logging
import os
import psutil
import datetime

#
# Logging
#

LOG_FORMAT = '%(asctime)-15s %(message)s'
logging.basicConfig(format=LOG_FORMAT)
log_level = 0
log_simple = False
log_destination = sys.stderr
process = None


def set_log_level(level, buffered=False):
    global log_level
    log_level = level
    logging.basicConfig(level=log_level * 10)
    if buffered:
        os.environ['PYTHONUNBUFFERED'] = ''
    else:
        os.environ['PYTHONUNBUFFERED'] = '1'


def set_log_destination(file):
    global log_destination
    log_destination = file


def get_logger(module):
    global log_level
    logger = logging.getLogger(module)
    logger.setLevel(log_level * 10)
    return logger


def log_generic(level_str, level, message, *msg_fmt, file=None, force=False):
    if file is None:
        file = log_destination
    if force or level >= log_level:
        global log_simple
        str_msg_fmt = [str(o) for o in msg_fmt]
        msg = message.format(*str_msg_fmt)
        if log_simple:
            print(level_str + ':', msg, file=file)
        else:
            print(level_str, '|', datetime.datetime.now(), '|', msg, file=file)


def trace(message, *msg_fmt, file=None, force=False, mem=False):
    global process
    if mem and process is None:
        process = psutil.Process(os.getpid())
    if mem:
        log_generic('TRACE', 0, message, *msg_fmt, '|', process.memory_info().rss / 1024 ** 2, 'MB',
                   file=file, force=force)
    else:
        log_generic('TRACE', 0, message, *msg_fmt, '|', file=file, force=force)


def debug(message, *msg_fmt, file=None, force=False, mem=False):
    if mem:
        log_generic('DEBUG', 1, message, *msg_fmt, process.memory_info().rss / 1024 ** 2, 'MB',
                   file=file, force=force)
    else:
        log_generic('DEBUG', 1, message, *msg_fmt, file=file, force=force)


def info(message, *msg_fmt, file=None, force=False):
    log_generic('INFO ', 2, message, *msg_fmt, file=file, force=force)


def warn(message, *msg_fmt, file=None, force=False):
    log_generic('WARN ', 3, message, *msg_fmt, file=file, force=force)


def error(message, *msg_fmt, file=None, force=False):
    log_generic('ERROR', 4, message, *msg_fmt, file=file, force=force)


def exception(message, e: Exception, *msg_fmt, file=None, force=False):
    import traceback
    log_generic('ERROR', 4, message + ': ' + str(e), *msg_fmt, file=file, force=force)
    traceback.print_exc()

#
#
#